﻿using Contracts.Generated.Flows.Schuldbemiddeling.v1;
using Infrastructure.Utils.Xml;
using SoapEndpoint.Service.Flows.Schuldbemiddeling.v1.ShvService;
using SoapEndpoint.Service.Proxy.Resources.Schuldbemiddeling.v1;
using SoapEndpoint.Tester.Infrastructure;
using BerichtHeaderSHV = SoapEndpoint.Service.Flows.Schuldbemiddeling.v1.ShvService.BerichtHeaderSHV;
using Betaalvoorstel = Contracts.Generated.Flows.Schuldbemiddeling.v1.Betaalvoorstel;
using MessageboxBericht = SoapEndpoint.Service.Systeem.v1.MessageBoxService.MessageboxBericht;
using MessageboxConfirmatieBericht = SoapEndpoint.Service.Systeem.v1.MessageBoxService.MessageboxConfirmatieBericht;
using Saldoverzoek = Contracts.Generated.Flows.Schuldbemiddeling.v1.Saldoverzoek;
using SaldoverzoekVorderingen = Contracts.Generated.Flows.Schuldbemiddeling.v1.SaldoverzoekVorderingen;
using ShvBericht = SoapEndpoint.Service.Flows.Schuldbemiddeling.v1.ShvService.ShvBericht;
using ShvKnooppuntFoutcode = Contracts.Generated.Flows.Schuldbemiddeling.v1.ShvKnooppuntFoutcode;

namespace SoapEndpoint.Tester;

public class CodeExamples
{
    private const string SenderCertificateThumbprint =
        "certificate thubmprint from certificate in certificate store";

    private static readonly Guid ReceiverReference = Guid.Parse("Get reference from address book");

    /// <summary>
    ///     Send message based on the Saldoverzoek class that was generated based on the xsds.
    /// </summary>
    public async Task Example1()
    {
        // Create the message
        var saldoVerzoek = new Saldoverzoek
        {
            KlantnummerSHV = "ShvKenmerk",
            Vorderingen = new SaldoverzoekVorderingen
            {
                new() { FactuurnummerSEVolgensSHV = "reference1" },
                new() { FactuurnummerSEVolgensSHV = "reference2" }
            }
            //TODO: initialize object
        };

        // Serialize the message to a string
        var xmlString = DataContractSerializeHelper.Serialize(saldoVerzoek);

        await SendSaldoverzoek(xmlString, ReceiverReference, SenderCertificateThumbprint);
    }

    /// <summary>
    ///     Send message based on saldoverzoek xml string in the resource file
    /// </summary>
    public async Task Example2()
    {
        await SendSaldoverzoek(Messages_v1_0.SHV_Saldoverzoek, ReceiverReference, SenderCertificateThumbprint);
    }

    public async Task Example3()
    {
        await GetMessages(SenderCertificateThumbprint);
    }

    /// <summary>
    ///     Send the message
    /// </summary>
    /// <param name="xmlString">The request xml string.</param>
    /// <param name="receiverReference">The receiver reference from the address book</param>
    /// <param name="senderCertificateThumbprint">The sender certificate thumbprint</param>
    public async Task SendSaldoverzoek(string xmlString, Guid receiverReference, string senderCertificateThumbprint)
    {
        // Validate the string against the XSD
        var validationResult = new XmlValidator().ValidateDocument<Saldoverzoek>(xmlString);

        if (!validationResult.Success)
        {
            throw new Exception(string.Join(",", validationResult.Errors));
        }

        var header = new BerichtHeaderSHV
        {
            Referentienummer = Guid.NewGuid(),
            ConversatieReferentienummer = Guid.NewGuid(),
            Ontvanger = receiverReference, // Reference from the address book
            CrossReferentienummer = null
        };

        // Load the certificate from the certificate store.
        var certificate = CertificateHelper.FindCertificateByThumbprint(senderCertificateThumbprint);

        var proxy = ServiceHostHelper_v1.GetShvProxy(certificate);

        var result = await proxy.SendMessageAsync(new SendMessageRequest
        {
            bericht = new ShvBericht
            {
                Content = xmlString,
                Berichttype = ShvBerichttype.SHV_Saldoverzoek.ToString()
            },
            Header = header
        });

        if (!result.SendMessageResult.Success)
        {
            throw new Exception(string.Join(",", result.SendMessageResult.Fouten));
        }
    }

    private async Task GetMessages(string senderCertificateThumbprint)
    {
        // Load the certificate from the certificate store.
        var certificate = CertificateHelper.FindCertificateByThumbprint(senderCertificateThumbprint);

        var proxy = ServiceHostHelper_v1.GetMessageBoxProxy(certificate);

        var messageInfo = await proxy.GetMessagesAsync();

        var handler = new MessageBoxMessageHandler();

        // The registers the ErrorMessageHandler method so it will be called when an error message is received from the messagebox.
        handler.AddHandler<ShvKnooppuntFoutbericht>(Berichttype.SYS_FoutBericht, ErrorMessagehandler);

        // The handlers below are empty
        handler.AddHandler<ShvKnooppuntAfleverbericht>(Berichttype.SYS_AfleverBericht, (e, m) => { });
        handler.AddHandler<Saldoverzoek>(Berichttype.SHV_Saldoverzoek, (e, m) => { });
        handler.AddHandler<Betaalvoorstel>(Berichttype.SHV_Betaalvoorstel, (e, m) => { });
        // Add more handler here.

        handler.OnMessageReceived(messageInfo);

        if (messageInfo?.BatchReferentie != null)
        {
            var message = new MessageboxConfirmatieBericht
            {
                BatchReferentie = messageInfo.BatchReferentie.Value
            };

            await proxy.ConfirmBatchAsync(message);
        }
    }

    /// <summary>
    ///     Example handler
    /// </summary>
    /// <param name="errorMessage">The message</param>
    /// <param name="message">The message box message</param>
    public void ErrorMessagehandler(ShvKnooppuntFoutbericht errorMessage, MessageboxBericht message)
    {
        foreach (var error in errorMessage.Fouten)
            if (error.Naam == ShvKnooppuntFoutcode.Berichtbodyisnietconformschema)
            {
                // Handle error
            }
    }
}